<?php
/**
* RDF Generator
*
* Klasse zur Generierung von RDF Channels
*
* Beispielanwendung
*
*
* Die Klasse enthlt eine Fehlerroutine. Fehler werden aber nur dann
* ausgegeben, wenn dieses Script direkt und nicht ber einen RSS-Parser
* aufgerufen wird.
* Der RSS-Parser wrde nur einen XML-Fehler melden, der mit der eigentlichen
* Ursache aber nichts zu tun haben mu.
*
*
* (c) 2002 Stefan Fischer
*
* @author  Stefan Fischer <post@stefan-fischer.net>
* @version 0.1
* @package RDFGenerator
* @access  public
*/
require_once 'class.rdf_generator.php';


/*
* neue Instanz des RDF Generators
*/
$rdf = new RDFGenerator;

/*
* XML Version setzen
*
* standardmig ist die Version auf 1.0 gesetzt
* Die Verwendung der Funktion ist OPTIONAL
*
*/
$rdf->setVersion('1.0');

/*
* XML Encoding setzen
*
* standardmig ist das Encoding auf 'UTF-8' gesetzt
* Die Verwendung der Funktion ist OPTIONAL
*
*/
$rdf->setEncoding('UTf-8');

/*
* XML Namespaces setzen
*
* Die Verwendung der Funktion ist OPTIONAL
*
* 'DC' 	-> wird automatisch hinzugefgt, wenn fr den Channel oder Items auch DublinCore Metadaten verwendet werden
*
*/
$rdf->setNamespaces(array('RDF'	=> 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
 						  'XML'	=> 'http://purl.org/rss/1.0/',
 						  'DC'	=> 'http://purl.org/dc/elements/1.1/'
 				 		));


/*
* neuen Channel einrichten
*
* Die Funktion fgt dem neuen Channel die Basisdaten hinzu
* 'title' und 'link' mssen hinzugefgt werden
*
* 'title'	-> Titel des Channels
* 'link'	-> URL auf den der Channel verweist
* 'dc'		-> fgt dem Channel OPTIONAL beliebige DublinCore Metadaten hinzu
*/
$rdf->createChannel(array('title'		=> 'Mein Testchannel',
						  'link'        => 'http://meine.domain.de',
						  'description'	=> 'Das ist die Beschreibung zu meinem Testchannel',
						  'dc'			=> array('date'		 => '2002-07-21T14:13:03+00:00',
						  						 'publisher' => 'Mein Name'
						  						 )
						));

/*
* fgt dem Channel Items hinzu
*
* 'title', 'link' mssen mssen bergeben werden
*
* 'title'	-> Titel des Items
* 'link'	-> URL auf den das Item verweist
* 'dc'		-> fgt einem Item OPTIONAL beliebige DublinCore Metadaten hinzu
*
*/
$items[] = array('title'       => 'Item eins',
				 'link'        => 'http://meine.domain.de/item/eins',
		         'description' => 'Beschreibung fuer Item eins',
				 'dc'		   => array('creator' => 'Mein Name')
				);

$items[] = array('title'       => 'Item zwei',
			     'link'        => 'http://meine.domain.de/item/zwei',
		         'description' => 'Beschreibung fuer Item zwei'
				);

$items[] = array('title'       => 'Item drei',
			     'link'        => 'http://meine.domain.de/item/drei',
		         'description' => 'Beschreibung fuer Item drei'
				);

$items[] = array('title'       => 'Item vier',
		 	     'link'        => 'http://meine.domain.de/item/vier',
		         'description' => 'Beschreibung fuer Item vier'
				);

for ($i = 0; $i < count($items); $i++)  {
	$rdf->addItem($items[$i]);
}

$rdf->addItem(array('title'       => 'Item & fuenf',
				    'link'        => 'http://meine.domain.de/item/fuenf',
		            'description' => 'Beschreibung fuer >>>>> Item fuenf'
				));


/*
* fgt dem Channel OPTIONAL ein Bild hinzu
*
* alle drei Angaben zu den Bilddaten mssen gemacht werden
*
* 'title'	-> Alttext des Bildes
* 'link'	-> URL, auf den das Bild verweist
* 'url'		-> Pfad zum Bild
*
*/
$rdf->addImage(array('title' => 'ein Bild',
					 'link'	 => 'http://meine.domain.de',
					 'url'	 => 'http://meine.domain.de/bild.gif'
					));


/*
* fgt dem Channel OPTIONAL ein Formular mit einem Inputfeld hinzu
*
* 'title', 'link' und 'name' mssen mssen bergeben werden
*
* 'title'	-> Titel ber dem Formular
* 'link'	-> form action
* 'name		-> Name des input feldes
*
*/
$rdf->addTextinput(array('title'	   => 'Ein Formular',
			   		 	 'link'		   => 'http://www.meine.domain.de/foo/bar.php',
			  		 	 'name'		   => 'foo',
			   		 	 'description' => 'Beschreibung fuer das Formular'
			));

/*
* Ausgabe
* Das Script darf ansonsten keinen weiteren Output enthalten
* Fertig ist der RDF Channel und kann unter dem URL des Scriptes ffentlich zugnglich gemacht werden
*
*/
$rdf->printRDF();

/*
*
* Die Funktion schreibt den fertigen RDF Channel in eine Datei, deren URL als neuer Newsfeed
* ebenfalls ffentlich angeboten werden kann
*
* Es mu der URL der Datei bergeben werden
*
*/

//$rdf->writeRDF('myrdf.rdf');


?>
